/**
  * color-fns v0.0.7
  * (c) 2018 Baianat
  * @license MIT
  */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.ColorFns = factory());
}(this, (function () { 'use strict';

  var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
    return typeof obj;
  } : function (obj) {
    return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
  };

  var classCallCheck = function (instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  };

  var createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  var get = function get(object, property, receiver) {
    if (object === null) object = Function.prototype;
    var desc = Object.getOwnPropertyDescriptor(object, property);

    if (desc === undefined) {
      var parent = Object.getPrototypeOf(object);

      if (parent === null) {
        return undefined;
      } else {
        return get(parent, property, receiver);
      }
    } else if ("value" in desc) {
      return desc.value;
    } else {
      var getter = desc.get;

      if (getter === undefined) {
        return undefined;
      }

      return getter.call(receiver);
    }
  };

  var inherits = function (subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }

    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
  };

  var possibleConstructorReturn = function (self, call) {
    if (!self) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return call && (typeof call === "object" || typeof call === "function") ? call : self;
  };

  function getColorModel(color) {
    if ((typeof color === 'undefined' ? 'undefined' : _typeof(color)) === 'object' && color.model) {
      return color.model;
    }

    if (color.slice(0, 1) === '#' && (color.length === 4 || color.length === 7)) {
      return 'hex';
    }

    if (color.slice(0, 1) === '#' && (color.length === 6 || color.length === 9)) {
      return 'hex';
    }

    if (color.slice(0, 4).toUpperCase() === 'RGBA') {
      return 'rgb';
    }

    if (color.slice(0, 3).toUpperCase() === 'RGB') {
      return 'rgb';
    }

    if (color.slice(0, 4).toUpperCase() === 'HSLA') {
      return 'hsl';
    }

    if (color.slice(0, 3).toUpperCase() === 'HSL') {
      return 'hsl';
    }

    return false;
  }

  function hexNumToDec(hexNum) {
    if (isNaN(parseInt(hexNum, 16))) {
      return 0;
    }

    return parseInt(hexNum, 16);
  }

  function isBetween(lb, ub) {
    return function (value) {
      return value >= lb && value <= ub;
    };
  }
  function getRandomInt(min, max) {
    return Math.floor(Math.random() * (max - min + 1) + min);
  }

  function mixValue(val1, val2) {
    var ratio = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0.5;

    return Number((val1 * (1 - ratio) + val2 * ratio).toFixed(2));
  }

  var Color = function () {
    function Color(components) {
      var _this = this;

      classCallCheck(this, Color);

      this.invalid = !this.validate(components);
      if (!this.invalid) {
        Object.keys(components).forEach(function (c) {
          _this[c] = components[c];
        });
      }
      this.init();
    }

    createClass(Color, [{
      key: 'init',
      value: function init() {}
    }, {
      key: 'validate',
      value: function validate(components) {
        return !!components && (typeof components === 'undefined' ? 'undefined' : _typeof(components)) === 'object';
      }
    }]);
    return Color;
  }();

  var RgbColor = function (_Color) {
    inherits(RgbColor, _Color);

    function RgbColor() {
      classCallCheck(this, RgbColor);
      return possibleConstructorReturn(this, (RgbColor.__proto__ || Object.getPrototypeOf(RgbColor)).apply(this, arguments));
    }

    createClass(RgbColor, [{
      key: 'validate',
      value: function validate(components) {
        if (!get(RgbColor.prototype.__proto__ || Object.getPrototypeOf(RgbColor.prototype), 'validate', this).call(this, components)) {
          return false;
        }

        var isInRange = isBetween(0, 255);

        return isInRange(components.red) && isInRange(components.green) && isInRange(components.blue);
      }
    }, {
      key: 'init',
      value: function init() {
        this.model = 'rgb';
        this.alpha = this.alpha === undefined ? 1 : this.alpha;
      }
    }, {
      key: 'toString',
      value: function toString() {
        if (this.invalid) {
          return 'Invalid Color';
        }
        if (isBetween(0, 0.999)(this.alpha)) {
          return 'rgba(' + this.red + ',' + this.green + ',' + this.blue + ',' + this.alpha + ')';
        }
        return 'rgb(' + this.red + ',' + this.green + ',' + this.blue + ')';
      }
    }]);
    return RgbColor;
  }(Color);

  var HslColor = function (_Color2) {
    inherits(HslColor, _Color2);

    function HslColor() {
      classCallCheck(this, HslColor);
      return possibleConstructorReturn(this, (HslColor.__proto__ || Object.getPrototypeOf(HslColor)).apply(this, arguments));
    }

    createClass(HslColor, [{
      key: 'validate',
      value: function validate(components) {
        if (!get(HslColor.prototype.__proto__ || Object.getPrototypeOf(HslColor.prototype), 'validate', this).call(this, components)) {
          return false;
        }

        var isPercentage = isBetween(0, 100);

        return isBetween(0, 360)(components.hue) && isPercentage(components.lum) && isPercentage(components.sat);
      }
    }, {
      key: 'init',
      value: function init() {
        this.model = 'hsl';
        this.alpha = this.alpha || 1;
      }
    }, {
      key: 'toString',
      value: function toString() {
        if (this.invalid) {
          return 'Invalid Color';
        }
        if (isBetween(0, 0.999)(this.alpha)) {
          return 'hsla(' + this.hue + ',' + this.sat + '%,' + this.lum + '%,' + this.alpha + ')';
        }
        return 'hsl(' + this.hue + ',' + this.sat + '%,' + this.lum + '%)';
      }
    }]);
    return HslColor;
  }(Color);

  var HexColor = function (_Color3) {
    inherits(HexColor, _Color3);

    function HexColor() {
      classCallCheck(this, HexColor);
      return possibleConstructorReturn(this, (HexColor.__proto__ || Object.getPrototypeOf(HexColor)).apply(this, arguments));
    }

    createClass(HexColor, [{
      key: 'validate',
      value: function validate(components) {
        if (!get(HexColor.prototype.__proto__ || Object.getPrototypeOf(HexColor.prototype), 'validate', this).call(this, components)) {
          return false;
        }

        return (/(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test('#' + components.red + components.green + components.blue)
        );
      }
    }, {
      key: 'init',
      value: function init() {
        this.model = 'hex';
        this.alpha = this.alpha || 'ff';
      }
    }, {
      key: 'toString',
      value: function toString() {
        if (this.invalid) {
          return 'Invalid Color';
        }
        if (isBetween(0, 0.999)(hexNumToDec(this.alpha) / 255)) {
          return '#' + this.red + this.green + this.blue + this.alpha;
        }
        return '#' + this.red + this.green + this.blue;
      }
    }]);
    return HexColor;
  }(Color);

  function parseRgb(rgb) {
    if ((typeof rgb === 'undefined' ? 'undefined' : _typeof(rgb)) === 'object') {
      return rgb;
    }

    // will consider rgb/rgba color prefix as a valid input color
    // while the output will be a valid web colors
    // valid input colors examples 'rgb(100, 0, 0, 0.5)', 'rgba(0, 0, 0)'
    // the output for the inputted examples 'rgba(100, 0, 0, 0.5)', 'rgb(0, 0, 0)'
    var match = rgb.match(/^rgba?\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*,*\s*(\d*(?:\.\d+)*)*\)/i);
    if (!match || match.length < 4) {
      return new RgbColor();
    }
    return new RgbColor({
      red: Number(match[1]),
      green: Number(match[2]),
      blue: Number(match[3]),
      alpha: Number(match[4])
    });
  }

  function expandHexShorthand(hex) {
    var regex = /^#([a-f\d])([a-f\d])([a-f\d])([a-f\d])*$/i;
    if ((hex.length === 5 || hex.length === 4) && regex.test(hex)) {
      hex = hex.replace(regex, function (m, r, g, b, a) {
        return '#' + r + r + g + g + b + b + (a ? '' + a + a : '');
      });
    }

    return hex;
  }

  function parseHex(hex) {
    if ((typeof hex === 'undefined' ? 'undefined' : _typeof(hex)) === 'object') {
      return hex;
    }

    var expanded = expandHexShorthand(hex);
    var match = expanded.match(/^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})*/i);
    if (!match || match.length < 4) {
      return new HexColor();
    }

    return new HexColor({
      hex: expanded,
      red: match[1],
      green: match[2],
      blue: match[3],
      alpha: match[4]
    });
  }

  function parseHsl(hsl) {
    if ((typeof hsl === 'undefined' ? 'undefined' : _typeof(hsl)) === 'object') {
      return hsl;
    }

    // will consider hsl/hsla color prefix as a valid input color
    // while the output will be a valid web colors
    // valid input colors examples 'hsl(255, 100%, 50%, 0.5)', 'hsla(100, 100%, 50%)'
    // the output for the inputted examples 'hsla(255, 100%, 50%, 0.5)', 'hsl(100, 100%, 50%)'
    var match = hsl.match(/^hsla?\(\s*(\d+)\s*,\s*(\d+)%\s*,\s*(\d+)%\s*,*\s*(\d*(?:\.\d+)*)*\)/i);
    if (!match || match.length < 4) {
      return new HslColor();
    }

    return new HslColor({
      hue: Number(match[1]),
      sat: Number(match[2]),
      lum: Number(match[3]),
      alpha: Number(match[4])
    });
  }

  /**
   * Checks if the given color string is valid (parsable).
   *
   * @param {String} color The color string to be checked.
   */
  function isValidColor(color) {
    var model = getColorModel(color);

    if (model === 'rgb') {
      return !parseRgb(color).invalid;
    }
    if (model === 'hex') {
      return !parseHex(color).invalid;
    }
    if (model === 'hsl') {
      return !parseHsl(color).invalid;
    }
    return false;
  }

  function isHexShorthand(hex) {
    var regex = /^#([a-f\d])([a-f\d])([a-f\d])([a-f\d])*$/i;
    if ((hex.length === 5 || hex.length === 4) && regex.test(hex)) {
      return true;
    }

    return false;
  }

  function decNumToHex(decNum) {
    decNum = Math.floor(decNum);
    if (isNaN(decNum)) {
      return '00';
    }

    return ('0' + decNum.toString(16)).slice(-2);
  }

  function rgbToHex(rgb) {
    if (!rgb) {
      return new HexColor();
    }
    rgb = parseRgb(rgb);
    var _ref = [decNumToHex(rgb.red), decNumToHex(rgb.green), decNumToHex(rgb.blue), rgb.alpha ? decNumToHex(rgb.alpha * 255) : null],
        rr = _ref[0],
        gg = _ref[1],
        bb = _ref[2],
        aa = _ref[3];


    return new HexColor({
      red: rr,
      green: gg,
      blue: bb,
      alpha: aa || 'ff'
    });
  }

  function rgb2Hsl(rgb) {
    if (!rgb) {
      return new HslColor();
    }

    rgb = parseRgb(rgb);

    // Convert the RGB values to the range 0-1
    var _ref = [rgb.red / 255, rgb.green / 255, rgb.blue / 255, rgb.alpha],
        red = _ref[0],
        green = _ref[1],
        blue = _ref[2],
        alpha = _ref[3];
    var hue = 0,
        sat = 0,
        lum = 0;

    // Find the minimum and maximum values of R, G and B.

    var min = Math.min(red, green, blue);
    var max = Math.max(red, green, blue);

    // Calculate the lightness value
    lum = (min + max) / 2;

    // Calculate the saturation.
    if (min !== max) {
      sat = lum > 0.5 ? (max - min) / (2 - max - min) : (max - min) / (max + min);
    }

    // calculate the hue
    if (red >= max && min !== max) {
      hue = 60 * ((green - blue) / (max - min));
    }
    if (green >= max && min !== max) {
      hue = 60 * (2.0 + (blue - red) / (max - min));
    }
    if (blue >= max && min !== max) {
      hue = 60 * (4.0 + (red - green) / (max - min));
    }

    // normalize values
    hue = hue < 0 ? Math.floor(hue + 360) : Math.floor(hue);
    sat = Math.floor(sat * 100);
    lum = Math.floor(lum * 100);

    return new HslColor({
      hue: hue,
      sat: sat,
      lum: lum,
      alpha: alpha
    });
  }

  function hexToRgb(hex) {
    if (!hex) {
      return new RgbColor();
    }

    var _parseHex = parseHex(hex),
        red = _parseHex.red,
        green = _parseHex.green,
        blue = _parseHex.blue,
        alpha = _parseHex.alpha;

    return new RgbColor({
      red: hexNumToDec(red),
      green: hexNumToDec(green),
      blue: hexNumToDec(blue),
      alpha: alpha === undefined ? 1 : Number((hexNumToDec(alpha) / 255).toFixed(2))
    });
  }

  function normalizeDecNum(value) {
    return Math.min(Math.max(parseInt(value), 0), 255);
  }

  function hslToRgb(hsl) {
    if (!hsl) {
      return new RgbColor();
    }
    hsl = parseHsl(hsl);

    var _ref = [hsl.hue / 360, hsl.sat / 100, hsl.lum / 100, hsl.alpha],
        hue = _ref[0],
        sat = _ref[1],
        lgh = _ref[2],
        alpha = _ref[3];
    var red = 0,
        green = 0,
        blue = 0;


    if (sat === 0) {
      red = green = blue = normalizeDecNum(lgh * 255);
    }

    if (sat !== 0) {
      var temp1 = lgh >= 50 ? lgh + sat - lgh * sat : lgh * (1 + sat);
      var temp2 = 2 * lgh - temp1;

      var testHue = function testHue(test) {
        if (test < 0) test += 1;
        if (test > 1) test -= 1;
        if (test < 1 / 6) return temp2 + (temp1 - temp2) * 6 * test;
        if (test < 1 / 2) return temp1;
        if (test < 2 / 3) return temp2 + (temp1 - temp2) * (2 / 3 - test) * 6;
        return temp2;
      };

      red = normalizeDecNum(255 * testHue(hue + 1 / 3));
      green = normalizeDecNum(255 * testHue(hue));
      blue = normalizeDecNum(255 * testHue(hue - 1 / 3));
    }

    return new RgbColor({
      red: red,
      green: green,
      blue: blue,
      alpha: alpha
    });
  }

  function hexToHsl(hex) {
    if (!hex) {
      return new HslColor();
    }
    return rgb2Hsl(hexToRgb(hex));
  }

  function hslToHex(hsl) {
    if (!hsl) {
      return new HexColor();
    }
    return rgbToHex(hslToRgb(hsl));
  }

  /**
   * Parses the given color string into a RGB color object.
   *
   * @param {String} color The color to be parsed and converted.
   */
  function toRgb(color) {
    var model = getColorModel(color);

    if (model === 'hex') {
      return hexToRgb(color);
    }

    if (model === 'hsl') {
      return hslToRgb(color);
    }

    if (model === 'rgb' && typeof color === 'string') {
      return parseRgb(color);
    }

    if (model === 'rgb' && (typeof color === 'undefined' ? 'undefined' : _typeof(color)) === 'object') {
      return color;
    }

    return new RgbColor();
  }

  /**
   * Parses the given color string into a HSL color object.
   *
   * @param {String} color The color to be parsed and converted.
   */
  function toHsl(color) {
    var model = getColorModel(color);

    if (model === 'hex') {
      return hexToHsl(color);
    }

    if (model === 'rgb') {
      return rgb2Hsl(color);
    }

    if (model === 'hsl' && typeof color === 'string') {
      return parseHsl(color);
    }

    if (model === 'hsl' && (typeof color === 'undefined' ? 'undefined' : _typeof(color)) === 'object') {
      return color;
    }

    return new HslColor();
  }

  /**
   * Parses the given color string into a Hex color object.
   *
   * @param {String} color The color to be parsed and converted.
   */
  function toHex(color) {
    var model = getColorModel(color);

    if (model === 'rgb') {
      return rgbToHex(color);
    }

    if (model === 'hsl') {
      return hslToHex(color);
    }

    if (model === 'hex' && typeof color === 'string') {
      return parseHex(color);
    }

    if (model === 'hex' && (typeof color === 'undefined' ? 'undefined' : _typeof(color)) === 'object') {
      return color;
    }

    return new HexColor();
  }

  function getRandomColor() {
    return 'rgb(' + getRandomInt(0, 255) + ', ' + getRandomInt(0, 255) + ', ' + getRandomInt(0, 255) + ')';
  }

  function alpha(color, alpha) {
    alpha = Number(alpha);
    if (isNaN(alpha) || alpha > 1 || alpha < 0) {
      return 'Invalid alpha';
    }

    var model = getColorModel(color);

    if (model === 'rgb') {
      var red = color.red,
          green = color.green,
          blue = color.blue;

      return new RgbColor({ red: red, green: green, blue: blue, alpha: alpha });
    }
    if (model === 'hex') {
      var _red = color.red,
          _green = color.green,
          _blue = color.blue;

      return new HexColor({ red: _red, green: _green, blue: _blue, alpha: decNumToHex(alpha * 255) });
    }
    if (model === 'hsl') {
      var hue = color.hue,
          sat = color.sat,
          lum = color.lum;

      return new HslColor({ hue: hue, sat: sat, lum: lum, alpha: alpha });
    }
    return 'Invalid color';
  }

  function mixColors(color1, color2, ratio) {
    color1 = toRgb(color1);
    color2 = toRgb(color2);
    var red = Math.floor(mixValue(color1.red, color2.red, ratio));
    var green = Math.floor(mixValue(color1.green, color2.green, ratio));
    var blue = Math.floor(mixValue(color1.blue, color2.blue, ratio));
    var alpha = mixValue(color1.alpha, color2.alpha, ratio);
    return new RgbColor({
      red: red,
      green: green,
      blue: blue,
      alpha: alpha
    });
  }

  var index = {
    getColorModel: getColorModel,
    isValidColor: isValidColor,
    isHexShorthand: isHexShorthand,
    hexNumToDec: hexNumToDec,
    decNumToHex: decNumToHex,
    rgbToHex: rgbToHex,
    rgbToHsl: rgb2Hsl,
    hexToRgb: hexToRgb,
    hslToRgb: hslToRgb,
    hexToHsl: hexToHsl,
    hslToHex: hslToHex,
    toRgb: toRgb,
    toHex: toHex,
    toHsl: toHsl,
    parseRgb: parseRgb,
    parseHex: parseHex,
    parseHsl: parseHsl,
    getRandomColor: getRandomColor,
    normalizeDecNum: normalizeDecNum,
    expandHexShorthand: expandHexShorthand,
    alpha: alpha,
    mixColors: mixColors,
    version: '0.0.7'
  };

  return index;

})));
